import { jsx as _jsx } from "react/jsx-runtime";
import { useCallback, useEffect, useMemo, useRef, useState, } from "react";
import ReactDOM from "react-dom";
import "./RightClickMenu.scss";
import PropTypes from "prop-types";
import classNames from "classnames";
import { calculateCoordinate } from "../functions/calculateCoordinate";
import MapLI from "../MapLI/MapLI";
/**
 * @component RightClickMenu
 * @author Xerxes (AmirHossein Salighedar) (https://github.com/amirho1)
 * @param param
 * @param param.rightClickTargets the className of target elements
 * @param param.preventDefaultOnWindow sets prevent from default behavior of right click
 * @param param.menuClassName
 * @param param.liClassName
 * @returns
 */
export default function RightClickMenu({ rightClickTargets, preventDefaultOnWindow = true, menuClassName, liClassName, }) {
    // get wrapper ul element of right click menu
    const rightClickUl = useRef(null);
    // is menu active or not boolean
    const [isActive, setIsActive] = useState(false);
    // coordinate of menu based on click
    const [menuCoordinate, setMenuCoordinate] = useState({
        left: "initial",
        top: "initial",
        right: "initial",
        bottom: "initial",
    });
    // func to clear the menu
    const clearMenuChildren = useCallback(() => {
        if (rightClickUl.current)
            ReactDOM.unmountComponentAtNode(rightClickUl.current);
    }, [rightClickUl]);
    // prevent from default behavior of right click
    useEffect(() => {
        if (preventDefaultOnWindow) {
            window.addEventListener("contextmenu", e => {
                e.preventDefault();
            });
        }
        // remove event listener
        return () => {
            window.removeEventListener("contextmenu", e => {
                e.preventDefault();
            });
        };
    }, [preventDefaultOnWindow]);
    const contextEventHandler = useCallback((e, target) => {
        var _a, _b;
        // prevent from default behavior on right click
        e.preventDefault();
        // stop the propagation
        e.stopPropagation();
        // display menu
        setIsActive(true);
        // set the coordinate of menu based on the right click coordinate
        setMenuCoordinate(calculateCoordinate({
            clientX: e.clientX,
            clientY: e.clientY,
            bottomBorderStartDistance: Number((_a = rightClickUl.current) === null || _a === void 0 ? void 0 : _a.clientWidth),
            rightBorderStartDistance: Number((_b = rightClickUl.current) === null || _b === void 0 ? void 0 : _b.clientWidth),
        }));
        // add elements to the menu
        ReactDOM.render(_jsx(MapLI, { menuList: target.menuList, liClassName: liClassName }, void 0), rightClickUl.current);
    }, []);
    // add event listener to all given targets
    useEffect(() => {
        if (rightClickTargets === null || rightClickTargets === void 0 ? void 0 : rightClickTargets.length)
            rightClickTargets.forEach(target => {
                const elements = document.querySelectorAll(`.${target.className}`);
                // clear menu before adding something in it
                clearMenuChildren();
                elements.forEach(element => element.addEventListener("contextmenu", e => contextEventHandler(e, target)));
            });
        // remove event listeners
        return () => {
            if (rightClickTargets === null || rightClickTargets === void 0 ? void 0 : rightClickTargets.length)
                rightClickTargets.forEach(target => {
                    const elements = document.querySelectorAll(`.${target.className}`);
                    // clear menu before adding something in it
                    clearMenuChildren();
                    elements.forEach(element => element.removeEventListener("contextmenu", e => contextEventHandler(e, target)));
                });
        };
    }, [rightClickTargets]);
    // add event listener to windows for closing the open menu on  click on every where of page
    useEffect(() => {
        const eventHandler = (e) => {
            e.stopPropagation();
            setIsActive(false);
        };
        window.addEventListener("click", eventHandler);
        window.addEventListener("contextmenu", eventHandler);
        //removing event listeners on unmounting
        return () => {
            window.removeEventListener("click", eventHandler);
            window.removeEventListener("contextmenu", eventHandler);
        };
    }, []);
    // styles
    const styles = useMemo(() => ({
        left: menuCoordinate.left,
        top: menuCoordinate.top,
        right: menuCoordinate.right,
        bottom: menuCoordinate.bottom,
    }), [menuCoordinate]);
    return (_jsx("ul", { ref: rightClickUl, style: styles, className: classNames("right-click-menu", { display: isActive }, menuClassName), "data-testid": "RightClickMenu" }, void 0));
}
RightClickMenu.propTypes = {
    rightClickTargets: PropTypes.arrayOf(PropTypes.shape({
        className: PropTypes.string.isRequired,
        menuList: PropTypes.arrayOf(PropTypes.oneOfType([PropTypes.string, PropTypes.element])).isRequired,
    })),
    preventDefaultOnWindow: PropTypes.bool,
    menuClassName: PropTypes.string,
    liCLassName: PropTypes.string,
};
//# sourceMappingURL=RightClickMenu.js.map